/*
 * Copyright (c) 2011-2016 Pivotal Software Inc, All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package reactor.core.publisher;

import java.util.function.Function;

import reactor.core.tuple.Tuple;
import reactor.core.tuple.Tuple2;

/**
 * @author Stephane Maldini
 * @since 2.0, 2.5
 */
final class Elapsed<T> implements Function<T, Tuple2<Long, T>> {

		long lastTime = System.currentTimeMillis();

		@Override
		public Tuple2<Long, T> apply(T ev) {
			long previousTime = lastTime;
			lastTime = System.currentTimeMillis();
			return Tuple.of(lastTime - previousTime, ev);
		}
}
